#include "errors_list.h"

//------------------------------------------------------------------------------
ErrorsList::ErrorsList(bec::DBObjectEditorBE* be)
           : _errors_tv(new Gtk::TreeView())
           , _errors_model(Gtk::ListStore::create(_errors_columns))
           , _be(be)
{
  _errors_tv->set_model(_errors_model);

  _errors_tv->append_column("Line"   , _errors_columns.lineno);
  _errors_tv->append_column("Message", _errors_columns.msg);

  _errors_tv->signal_cursor_changed().connect(sigc::mem_fun(*this, &ErrorsList::error_selected));
  _be->set_sql_parser_err_cb(sigc::mem_fun(this, &ErrorsList::add_error));
}

//------------------------------------------------------------------------------
ErrorsList::~ErrorsList()
{
  delete _errors_tv;
}

//------------------------------------------------------------------------------
void ErrorsList::switch_be(bec::DBObjectEditorBE* be)
{
  _be = be;
  _be->set_sql_parser_err_cb(sigc::mem_fun(this, &ErrorsList::add_error));
}

//------------------------------------------------------------------------------
void ErrorsList::clear_list()
{
  _errors_model->clear();
}

//------------------------------------------------------------------------------
int ErrorsList::add_error(const int line, const int err_tok_line_pos, const int err_tok_len, const std::string &err_msg)
{
  Gtk::TreeModel::Row row = *(_errors_model->append());
  std::string msg(err_msg);
  std::replace(msg.begin(), msg.end(), '\n', ' ');
  row[_errors_columns.lineno]    = line;
  row[_errors_columns.msg]       = msg;
  
  _sql_parser_err_cb(line, err_tok_line_pos, err_tok_len, err_msg);
  
  return 0;
}
    
//------------------------------------------------------------------------------
Gtk::TreeView& ErrorsList::widget()
{
  return *_errors_tv;
}

//------------------------------------------------------------------------------
void ErrorsList::error_selected()
{
  Gtk::TreeModel::Path   path;
  Gtk::TreeView::Column *column(0);
  
  _errors_tv->get_cursor(path, column);

  Gtk::TreeModel::Row row = *(_errors_model->get_iter(path));

  const int         lineno = row[_errors_columns.lineno];
  const std::string msg    = row[_errors_columns.msg];
  _error_selected_signal.emit(lineno, msg);
}







//------------------------------------------------------------------------------
TrackedPane::TrackedPane(Gtk::Paned* pane, bec::DBObjectEditorBE* be)
            : _pane(pane)
            , _pane_position(0)
            , _be(be)
{
  //_pane->set_position(_pane_position);
  _pane_pos_changed_conn = _pane->property_position().signal_changed().connect(sigc::mem_fun(this, &TrackedPane::track_pane));

  _be->set_sql_parser_log_cb(sigc::mem_fun(this, &TrackedPane::editor_parse_log_cb));
}

//------------------------------------------------------------------------------
void TrackedPane::switch_be(bec::DBObjectEditorBE* be)
{
  _be = be;
  _be->set_sql_parser_log_cb(sigc::mem_fun(this, &TrackedPane::editor_parse_log_cb));
}

//------------------------------------------------------------------------------
void TrackedPane::show2nd(const int default_2nd_size)
{
  if ( !_pane_position )
  {
    int size = _pane->get_height();
    _pane_position = size - default_2nd_size;
  }

  _pane_pos_changed_conn.block();
  _pane->set_position(_pane_position);
  _pane_pos_changed_conn.unblock();
}

//------------------------------------------------------------------------------
void TrackedPane::max1st()
{
  const int max_position = _pane->property_max_position();
  _pane_pos_changed_conn.block();
  _pane->set_position(max_position);
  _pane_pos_changed_conn.unblock();
}

//------------------------------------------------------------------------------
void TrackedPane::track_pane()
{
  _pane_position = _pane->get_position();
}

//------------------------------------------------------------------------------
void TrackedPane::editor_parse_log_cb(const std::vector<std::string>& list)
{
  //if ( list.size() > 0 )
  //  show2nd(80);
  //else
  //  max1st();
}
