/* 
 * © 2008-2009 Sun Microsystems, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/**
 * Implementation of the mforms view, which is the base for most of the visual controls in mforms.
 */

#include "stdafx.h"

#include <mforms/view.h>
#include <mforms/mforms.h>

using namespace mforms;

//--------------------------------------------------------------------------------------------------

View::View()
{
  _parent= 0;
  _view_impl= &ControlFactory::get_instance()->_view_impl;
  _resize_mode= ResizeBoth;
}

//--------------------------------------------------------------------------------------------------

AutoResizeMode View::get_resize_mode()
{ 
  return _resize_mode;
}

//--------------------------------------------------------------------------------------------------

void View::set_resize_mode(AutoResizeMode mode)
{
  _resize_mode= mode;
}

//--------------------------------------------------------------------------------------------------

void View::set_name(const std::string &name)
{
  _name= name;

  // Optional implementation.
  if (_view_impl->set_name)
    _view_impl->set_name(this, name);
}

//--------------------------------------------------------------------------------------------------

void View::set_tooltip(const std::string &text)
{
  _view_impl->set_tooltip(this, text);
}

//--------------------------------------------------------------------------------------------------

std::string View::get_name()
{ 
  return _name; 
}

//--------------------------------------------------------------------------------------------------

void View::set_parent(View *parent)
{
  _parent= parent;
}

//--------------------------------------------------------------------------------------------------

View* View::get_parent() const
{
  return _parent; 
}

//--------------------------------------------------------------------------------------------------

int View::get_width()
{
  return (*_view_impl->get_width)(this);
}

//--------------------------------------------------------------------------------------------------

int View::get_height()
{
  return (*_view_impl->get_height)(this);
}

//--------------------------------------------------------------------------------------------------

int View::get_preferred_width()
{
  return (*_view_impl->get_preferred_width)(this);
}

//--------------------------------------------------------------------------------------------------

int View::get_preferred_height()
{
  return (*_view_impl->get_preferred_height)(this);
}

//--------------------------------------------------------------------------------------------------

int View::get_x()
{
  return (*_view_impl->get_x)(this);
}

//--------------------------------------------------------------------------------------------------

int View::get_y()
{
  return (*_view_impl->get_y)(this);
}

//--------------------------------------------------------------------------------------------------

void View::set_position(int x, int y)
{
  (*_view_impl->set_position)(this, x, y);
}

//--------------------------------------------------------------------------------------------------

void View::set_size(int width, int height)
{
  (*_view_impl->set_size)(this, width, height);
}

//--------------------------------------------------------------------------------------------------

void View::show(bool flag)
{
  (*_view_impl->show)(this, flag);
}

//--------------------------------------------------------------------------------------------------

bool View::is_shown()
{
  return (*_view_impl->is_shown)(this);
}

//--------------------------------------------------------------------------------------------------

void View::set_enabled(bool flag)
{
  (*_view_impl->set_enabled)(this, flag);
}

//--------------------------------------------------------------------------------------------------

void View::needs_resize()
{
  if (_parent)
    _parent->needs_resize();
}

//--------------------------------------------------------------------------------------------------

void View::set_layout_dirty()
{
  (*_view_impl->set_layout_dirty)(this);
  if (_parent != NULL)
    _parent->set_layout_dirty();
}

//--------------------------------------------------------------------------------------------------

void View::relayout()
{
  if (_parent != NULL)
    _parent->relayout();
  (*_view_impl->relayout)(this);
}

//--------------------------------------------------------------------------------------------------

void View::on_resize()
{
  needs_resize();
  _signal_resized.emit();
}

//--------------------------------------------------------------------------------------------------

