#ifndef _MYSQL_SQL_NORMALIZER_H_
#define _MYSQL_SQL_NORMALIZER_H_


#include "mysql_sql_parser_base.h"
#include "grtsqlparser/sql_normalizer.h"


using namespace grt;


/** Implements DBMS specifics.
 * 
 * @ingroup sqlparser
 */
class MYSQL_SQL_PARSER_PUBLIC_FUNC Mysql_sql_normalizer : protected Mysql_sql_parser_base, public Sql_normalizer
{
public:
  Mysql_sql_normalizer();
  virtual ~Mysql_sql_normalizer() {}

  std::string normalize(const std::string &sql, const std::string &schema_name);

protected:
  // higher level
  int process_sql_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_statement(const MyxSQLTreeItem *tree);

  // parse tree core
  Parse_result process_create_schema_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_table_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_index_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_view_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_trigger_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_routine_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_server_link_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_tablespace_statement(const MyxSQLTreeItem *tree);
  Parse_result process_create_logfile_group_statement(const MyxSQLTreeItem *tree);
  Parse_result process_insert_statement(const MyxSQLTreeItem *tree);

  // parse tree helpers
  void qualify_obj_ident(const MyxSQLTreeItem *sp_name);
  void append_stmt_to_script(const std::string &stmt);

  // context
  std::string _schema_name;
  std::string _norm_stmt;
  std::string _norm_script;
  std::string _common_sql;
  int _cut_sym_count;

  class Null_state_keeper : Mysql_sql_parser_base::Null_state_keeper
  {
  public:
    Null_state_keeper(Mysql_sql_normalizer *sql_parser)
      : Mysql_sql_parser_base::Null_state_keeper(sql_parser),
        _sql_parser(sql_parser)
    {}
    ~Null_state_keeper()
    {
      _sql_parser->_schema_name= std::string();
      _sql_parser->_norm_stmt= std::string();
      _sql_parser->_norm_script= std::string();
      _sql_parser->_common_sql= std::string();
    }
  private:
    Mysql_sql_normalizer *_sql_parser;
  };
  friend class Null_state_keeper;
};


#endif // _MYSQL_SQL_NORMALIZER_H_
