#include "stdafx.h"

#include "mysql_sql_syntax_check.h"
#include "mysql_sql_parser_fe.h"


using namespace grt;


Mysql_sql_syntax_check::Null_state_keeper::~Null_state_keeper()
{
  _sql_parser->_check_sql_statement_syntax.disconnect();
}
#define NULL_STATE_KEEPER Null_state_keeper _nsk(this);


Mysql_sql_syntax_check::Mysql_sql_syntax_check()
{
  NULL_STATE_KEEPER
}


int Mysql_sql_syntax_check::check_sql_syntax(const std::string &sql)
{
  NULL_STATE_KEEPER
  _messages_enabled= true;
  _use_delimiter= false;
  int err_count= check_sql_statement_syntax(sql, sigc::mem_fun(this,
    &Mysql_sql_syntax_check::do_check_sql_syntax));
  return (err_count ? 0 : 1);
}


int Mysql_sql_syntax_check::check_trigger_syntax(const std::string &sql)
{
  NULL_STATE_KEEPER
  _messages_enabled= false;
  _use_delimiter= true;
  int err_count= check_sql_statement_syntax(sql, sigc::mem_fun(this,
    &Mysql_sql_syntax_check::do_check_trigger_syntax));
  return (err_count ? 0 : 1);
}


int Mysql_sql_syntax_check::check_view_syntax(const std::string &sql)
{
  NULL_STATE_KEEPER
  _messages_enabled= false;
  _use_delimiter= true;
  int err_count= check_sql_statement_syntax(sql, sigc::mem_fun(this,
    &Mysql_sql_syntax_check::do_check_view_syntax));
  return (err_count ? 0 : 1);
}


int Mysql_sql_syntax_check::check_routine_syntax(const std::string &sql)
{
  NULL_STATE_KEEPER
  _messages_enabled= false;
  _use_delimiter= true;
  int err_count= check_sql_statement_syntax(sql, sigc::mem_fun(this,
    &Mysql_sql_syntax_check::do_check_routine_syntax));
  return (err_count ? 0 : 1);
}


int Mysql_sql_syntax_check::check_sql_statement_syntax(const std::string &sql, Check_sql_statement_syntax check_sql_statement_syntax)
{
  _check_sql_statement_syntax= check_sql_statement_syntax;
  _process_sql_statement= sigc::mem_fun(this, &Mysql_sql_syntax_check::process_sql_statement);

  Mysql_sql_parser_fe sql_parser_fe;
  sql_parser_fe.ignore_dml= false;

  // set delimiter for sql script if needed
  std::string sql_= (_use_delimiter ? "DELIMITER //\n" : "") + sql + (_use_delimiter ? "\n//" : "");

  return parse_sql_script(sql_parser_fe, sql_.c_str());
}


int Mysql_sql_syntax_check::process_sql_statement(const MyxSQLTreeItem *tree)
{
  if (!tree)
  {
    log_syntax_error(_lineno, true, _err_tok_line_pos, _err_tok_len, _err_msg, 2);
    return 1;
  }

  if (tree)
    tree= tree->subitem(sql::_statement, sql::_create);

  if (!tree)
    return 1; //! select query shouldn't return error here

  if (pr_processed == _check_sql_statement_syntax(tree))
    return 0;
  else
    return 1;
}


Mysql_sql_parser_base::Parse_result Mysql_sql_syntax_check::do_check_sql_syntax(const MyxSQLTreeItem *tree)
{
  if (tree)
    return pr_processed;
  else
    return pr_irrelevant;
}

	
Mysql_sql_parser_base::Parse_result Mysql_sql_syntax_check::do_check_trigger_syntax(const MyxSQLTreeItem *tree)
{
  const MyxSQLTreeItem *trigger_tail= NULL;
  {
    static sql::symbol path1[]= { sql::_view_or_trigger_or_sp_or_event, sql::_definer_tail, sql::_ };
    static sql::symbol path2[]= { sql::_view_or_trigger_or_sp_or_event, sql::_no_definer_tail, sql::_ };
    static sql::symbol * paths[]= { path1, path2 };

    trigger_tail= tree->search_by_paths(paths, ARR_CAPACITY(paths));
    if (trigger_tail)
      trigger_tail= trigger_tail->subitem(sql::_trigger_tail);
  }

  if (trigger_tail && trigger_tail->subseq(sql::_TRIGGER_SYM))
    return pr_processed;
  else
    return pr_irrelevant;
}


Mysql_sql_parser_base::Parse_result Mysql_sql_syntax_check::do_check_view_syntax(const MyxSQLTreeItem *tree)
{
  const MyxSQLTreeItem *view_tail= NULL;
  {
    static sql::symbol path1[]= { sql::_view_or_trigger_or_sp_or_event, sql::_definer_tail, sql::_ };
    static sql::symbol path2[]= { sql::_view_or_trigger_or_sp_or_event, sql::_no_definer_tail, sql::_ };
    static sql::symbol path3[]= { sql::_view_or_trigger_or_sp_or_event, sql::_ };
    static sql::symbol * paths[]= { path1, path2, path3 };

    view_tail= tree->search_by_paths(paths, ARR_CAPACITY(paths));
    if (view_tail)
      view_tail= view_tail->subitem(sql::_view_tail);
  }

  if (view_tail)
    return pr_processed;
  else
    return pr_irrelevant;
}


Mysql_sql_parser_base::Parse_result Mysql_sql_syntax_check::do_check_routine_syntax(const MyxSQLTreeItem *tree)
{
  const MyxSQLTreeItem *routine_tail= NULL;
  {
    static sql::symbol path1[]= { sql::_view_or_trigger_or_sp_or_event, sql::_definer_tail, sql::_ };
    static sql::symbol path2[]= { sql::_view_or_trigger_or_sp_or_event, sql::_no_definer_tail, sql::_ };
    static sql::symbol * paths[]= { path1, path2 };

    routine_tail= tree->search_by_paths(paths, ARR_CAPACITY(paths));
    if (routine_tail)
    {
      static sql::symbol path1[]= { sql::_sp_tail, sql::_ };
      static sql::symbol path2[]= { sql::_sf_tail, sql::_ };
      static sql::symbol * paths[]= { path1, path2 };

      routine_tail= routine_tail->search_by_paths(paths, ARR_CAPACITY(paths));
    }
  }

  if (routine_tail)
    return pr_processed;
  else
    return pr_irrelevant;
}
