#include "mysql_table_editor_fe.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"

#include "mysql_table_editor_trigger_page.h"

//------------------------------------------------------------------------------
DbMySQLTableEditorTriggerPage::DbMySQLTableEditorTriggerPage(DbMySQLTableEditor *owner
                                                          ,MySQLTableEditorBE *be
                                                          ,MGGladeXML         *xml)
                             : _owner(owner)
                             , _be(be)
                             , _xml(xml)
                             , _errors(be)
{
  Gtk::VBox* trigger_code_win;
  _xml->get("trigger_code_holder", &trigger_code_win);
  _trigger_code.widget().set_size_request(-1, 100);
  trigger_code_win->add(_trigger_code.widget());
  _owner->add_sqleditor_text_change_timer(&_trigger_code, sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::parse_sql));
  _errors.set_sql_parser_err_cb(sigc::mem_fun(&_trigger_code, &SqlEditorFE::process_syntax_error));
  trigger_code_win->resize_children();
  
  Gtk::ScrolledWindow* win;
  _xml->get("trigger_parse_errors_holder", &win);
  win->add(_errors.widget());
  _errors.signal_error_selected().connect(sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::error_selected));

  Gtk::Paned *paned(0);
  _xml->get("trigger_paned", &paned);
  _pane = new TrackedPane(paned, be);

  _trigger_code.set_text(_be->get_all_triggers_sql());
}

//------------------------------------------------------------------------------
DbMySQLTableEditorTriggerPage::~DbMySQLTableEditorTriggerPage()
{
  delete _pane;
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::switch_be(MySQLTableEditorBE* be)
{
  _be = be;
  _pane->switch_be(_be);
  _trigger_code.set_text(_be->get_all_triggers_sql());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::refresh()
{
  _trigger_code.set_text(_be->get_all_triggers_sql());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::parse_sql(const std::string& sql)
{
  _pane->max1st();

  _errors.clear_list();
  _trigger_code.reset_syntax_errors();
  _be->set_triggers_sql(_trigger_code.get_text());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::error_selected(const int lineno, const std::string& msg)
{
  _trigger_code.scroll_to(lineno, msg);
}
